@@ -47,6 +47,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "dataset.h"
 #include "trie.h"
 #include "associative_list.h"
+#include "hy_string_buffer.h"
 
 #include <stdio.h>
 
@@ -170,7 +171,7 @@ public:
     /** Advance program counter */
     void      advance (void) {currentCommand ++;}
     
-    bool      is_compiled (long idx = -1) const {if (cli) if (idx < 0L) return true; else return cli->is_compiled[idx]; return false;}
+    bool      is_compiled (long idx = -1) const {if (cli) {if (idx < 0L) return true; else return cli->is_compiled[idx];} return false;}
     
     void      CopyCLIToVariables (void);
   
@@ -285,12 +286,14 @@ public:
     bool      HandleDoSQL                           (_ExecutionList&);
     bool      HandleFscanf                          (_ExecutionList&, bool is_sscanf = false);
     bool      HandleChoiceList                      (_ExecutionList&);
-  
+    bool      HandleInitializeIterator              (_ExecutionList&);
+    bool      HandleAdvanceIterator                 (_ExecutionList&);
+    
     long      get_code                              (void) const { return code; };
     unsigned  long parameter_count                  (void) const { return parameters.countitems();}
     
-    static  const _String   FindNextCommand       (_String&);
-    // finds & returns the next command block in input
+    static    void      FindNextCommand       (_String&, _StringBuffer&);
+    // finds & stores the next command from _String into _StringBuffer
     // chops the input to remove the newly found line
 
     static  long      ExtractConditions     (_String const& , long , _List&, char delimeter = ';', bool includeEmptyConditions = true);
@@ -424,7 +427,7 @@ protected:
     friend  class     _ExecutionList;
     friend  void      DeleteVariable     (long, bool);
     friend  void      UpdateChangingFlas (long);
-    friend  void      UpdateChangingFlas (_SimpleList&);
+    friend  void      UpdateChangingFlas (_SimpleList const&);
 
 
 private:
@@ -690,8 +693,9 @@ void    RetrieveModelComponents      (long, _Variable*&, _Variable*&, bool &);
 void    ReadModelList                (void);
 _String ProcessStringArgument        (_String* data);
 
-const _String _hblCommandAccessor          (_ExecutionList*, long);
-_String _HYGenerateANameSpace             (void);
+const _String _hblCommandAccessor               (_ExecutionList*, long);
+_String const _HYGenerateANameSpace             (void);
+void          _HYClearANameSpace                (_String const&);
 
 HBLObjectRef
 ProcessAnArgumentByType      (_String const*, _VariableContainer const*, long, _ExecutionList* = nil);
