@@ -5,9 +5,9 @@ HyPhy - Hypothesis Testing Using Phylogenies.
 Copyright (C) 1997-now
 Core Developers:
   Sergei L Kosakovsky Pond (spond@ucsd.edu)
-  Art FY Poon    (apoon@cfenet.ubc.ca)
+  Art FY Poon    (apoon42@uwo.ca)
   Steven Weaver (sweaver@ucsd.edu)
-  
+
 Module Developers:
 	Lance Hepler (nlhepler@gmail.com)
 	Martin Smith (martin.audacis@gmail.com)
@@ -42,13 +42,12 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 //#pragma once
 #include "category.h"
-#include "calcnode.h"
+#include "tree.h"
+#include "associative_list.h"
+#include "matrix.h"
+#include "vector.h"
 
-#ifdef __HYALTIVEC__
-#define   A_LARGE_NUMBER          1.e35
-#else
-#define   A_LARGE_NUMBER          1.e100
-#endif
+//#define   NUMERICAL_INFINITY          INFINITY
 
 #define   DEFAULTPARAMETERLBOUND  0.0
 #define   DEFAULTPARAMETERUBOUND  10000.0
@@ -101,7 +100,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 /* interval mapping functions */
 
 #define   _hyphyIntervalMapID                           0x0 // identify
-#define   _hyphyIntervalMapExpit                        0x1 // expit -- maps [-infty,infty) to [0,1] 
+#define   _hyphyIntervalMapExpit                        0x1 // expit -- maps [-infty,infty) to [0,1]
 // 1/(1+exp[-x])
 #define   _hyphyIntervalMapSqueeze                      0x2 // maps [0,infty) to [0,1)
 // x / (1+x)
@@ -133,8 +132,7 @@ enum _LikelihoodFunctionCountType {
 
 //_______________________________________________________________________________________
 
-class   _LikelihoodFunction: public BaseObj
-{
+class   _LikelihoodFunction: public BaseObj {
 
 public:
 
@@ -142,7 +140,8 @@ public:
 
     _LikelihoodFunction (void); // default - doesn't do much
     _LikelihoodFunction (_String&, _VariableContainer*); // from triplets
-    _LikelihoodFunction (_LikelihoodFunction&); // stack copy
+    _LikelihoodFunction (_LikelihoodFunction const&); // stack copy
+    const _LikelihoodFunction & operator = (_LikelihoodFunction const&);
     void        Init      (void);
 
     bool        Construct (_List&,_VariableContainer*);
@@ -153,9 +152,9 @@ public:
 
     virtual BaseRef     toStr (unsigned long = 0UL);
 
-    virtual BaseRef     makeDynamic (void);      // dynamic copy of this object
+    virtual BaseRef     makeDynamic (void) const;      // dynamic copy of this object
 
-    virtual void        Duplicate (BaseRef);         // duplicate an object into this one
+    virtual void        Duplicate (BaseRefConst);         // duplicate an object into this one
 
     _SimpleList const &GetIndependentVars (void) const; // return a list of all indepenent variables
     _SimpleList const &GetDependentVars   (void) const; // get all dependent vars of this object
@@ -170,19 +169,20 @@ public:
     // 4 - category variables
 
 
-    _Parameter  GetIthIndependent           (long, bool = true) const;     // get the value of i-th independent variable
+    hyFloat  GetIthIndependent           (long, bool = true) const;     // get the value of i-th independent variable
     const _String*  GetIthIndependentName           (long) const;     // get the name of i-th independent variable
-    _Parameter  GetIthDependent             (long) const;     // get the value of i-th dependent variable
+    const _String*  GetIthDependentName           (long) const;     // get the name of i-th independent variable
+    hyFloat  GetIthDependent             (long) const;     // get the value of i-th dependent variable
     void        GetAllIndependent           (_Matrix&) const; // store all indepenent values in a matrix
     _Variable*  GetIthIndependentVar        (long) const;     // get the variable object of i-th independent variable
     _Variable*  GetIthDependentVar          (long) const;     // get the variable object of i-th dependent variable
     _CategoryVariable*  GetIthCategoryVar           (long) const;     // get the variable object of i-th category variable
-    _Parameter  GetIthIndependentBound      (long, bool isLower = true) const;
+    hyFloat  GetIthIndependentBound      (long, bool isLower = true) const;
     // get the lower / upper bound for the i-th indepdendent variable
 
-    void        SetIthIndependent (long, _Parameter);           // set the value of i-th independent variable
-    bool        CheckAndSetIthIndependent (long, _Parameter);   // set the value of i-th independent variable
-    void        SetIthDependent           (long, _Parameter);   // set the value of i-th dependent variable
+    hyFloat        DerivativeCorrection (long, hyFloat) const;     // obtain the g'(x) for the chain rule differentiation under transformed variables
+    void        SetIthIndependent (long, hyFloat);           // set the value of i-th independent variable
+    bool        CheckAndSetIthIndependent (long, hyFloat);   // set the value of i-th independent variable
     bool        IsIthParameterGlobal      (long) const;
 
     long        SetAllIndependent         (_Matrix*);
@@ -190,24 +190,23 @@ public:
 
     void        UpdateIndependent (long,bool,_SimpleList* = nil,_SimpleList* = nil);
     void        UpdateDependent (long);
-    _AssociativeList*
-    SimulateCodonNeutral (_Matrix*, _Matrix*, long);
+    void        UpdateDependent (_AVLList const &);
 
     bool        PreCompute      (void);
     void        PostCompute     (void);
     virtual
-    _Parameter  Compute         (void);
+    hyFloat     Compute         (void);
 
     void        PrepareToCompute (bool = false);
     void        DoneComputing    (bool = false);
     virtual
-    _Matrix*    Optimize ();
-    _Matrix*    ConstructCategoryMatrix     (const _SimpleList&, char, bool = true, _String* = nil);
+    _Matrix*    Optimize (_AssociativeList const* options = nil);
+    _Matrix*    ConstructCategoryMatrix     (const _SimpleList&, unsigned, bool = true, _String* = nil);
 
-    _Parameter  SimplexMethod               (_Parameter& precision);
-    void        Anneal                      (_Parameter& precision);
+    hyFloat     SimplexMethod               (hyFloat& precision, unsigned long max_iterations = 100000UL, unsigned long max_evals = 0xFFFFFF);
+    void        Anneal                      (hyFloat& precision);
 
-    void        Simulate                    (_DataSet &,_List&, _Matrix* = nil, _Matrix* = nil, _Matrix* = nil, _String* = nil) const;
+    void        Simulate                    (_DataSet &,_List&, _Matrix* = nil, _Matrix* = nil, _Matrix* = nil, _String const* = nil) const;
 
     void        ReconstructAncestors        (_DataSet &, _SimpleList&, _String&, bool = false, bool = false, bool = false);
     // 20090224: added an argument to allow the marginal state reconstruction
@@ -215,13 +214,13 @@ public:
 
     long        MaximumDimension            (void);
 
-    virtual _PMathObj   CovarianceMatrix            (_SimpleList* = nil);
+    virtual HBLObjectRef   CovarianceMatrix            (_SimpleList* = nil);
 
     // compute  covariance matrix  based on the Hessian
     // optional list of parameters to estimate the conditional covariance for
 
 
-    virtual     void        RescanAllVariables      (void);
+    virtual     void        RescanAllVariables      (bool obtain_variable_mapping = false);
 
     long        DependOnTree            (_String const&) const;
     long        DependOnModel           (_String const&) const;
@@ -237,33 +236,29 @@ public:
     FindCategoryVar         (long);
     // return the category variable for a given partition
     void        RankVariables           (_AVLListX* tagger = nil);
-  
-  
+
+
     _TheTree*         GetIthTree            (long) const;
     _DataSetFilter const *  GetIthFilter    (long) const;
     _DataSetFilter *  GetIthFilterMutable   (long) const;
     _String const*         GetIthFilterName      (long) const;
-  
+
     _Matrix*          GetIthFrequencies     (long) const;
     _String const*         GetIthFrequenciesName (long) const;
- 
-  
+
+
     void        FillInConditionals      (long = -1);
-    
+
     void        Setup                   (bool check_reversibility = true);
-    bool&       HasBeenOptimized (void) {
-        return hasBeenOptimized;
-    }
-    void        ComputePruningEfficiency(long&, long&);
 
     long        SequenceCount           (long);
     unsigned long        SiteCount               (void) const;
-    void        Rebuild                 (bool = false);
-    virtual void        SerializeLF             (_String&, char=0, _SimpleList* = nil, _SimpleList* = nil);
+    void        Rebuild                 (bool rescan_parameters = false);
+    virtual void        SerializeLF             (_StringBuffer&, char=0, _SimpleList* = nil, _SimpleList* = nil);
     _Formula*   HasComputingTemplate    (void) const{
         return computingTemplate;
     }
-    void        StateCounter            (long);
+    void        StateCounter            (long) const;
     void        MPI_LF_Compute          (long, bool = false);
 
 #if defined _SLKP_LFENGINE_REWRITE_
@@ -284,7 +279,7 @@ public:
 #endif
 #endif
 
-    bool            ProcessPartitionList        (_SimpleList&, _Matrix*, _String const&) const;
+    bool            ProcessPartitionList        (_SimpleList&, _Matrix*) const;
     // given a matrix argument (argument 2; can be nil to include all)
     // populate a sorted list (argument 1)
     // of partitions indexed in the matrix (e.g. {{1,3}} would include the 2-nd and 4-th partitions
@@ -314,10 +309,14 @@ public:
     */
 
     _AssociativeList*CollectLFAttributes         (void) const;
-    void             UnregisterListeners         (void);
+    void    UnregisterListeners                  (void);
+    void    DetermineLocalUpdatePolicy           (void);
+    void    FlushLocalUpdatePolicy               (void);
+    void   _TerminateAndDump           (const _String& error, bool sig_term = false);
+
 protected:
-  
-  
+
+
     void            AllocateTemplateCaches  (void);
     bool            CheckIthPartition       (unsigned long index, _String* errorString, _String const* = nil, _String const * = nil, _String const * = nil);
 
@@ -328,8 +327,8 @@ protected:
     // 20090211: A utility function to reset site results.
     void            GetInitialValues        (void) const;
     bool            checkPermissibility     (_Matrix&m, long row);
-    _Parameter      computeAtAPoint         (_Matrix&m, long row = 0);
-    _Parameter      replaceAPoint           (_Matrix&m, long row, _Matrix&p, _Parameter& nV, _Matrix& fv);
+    hyFloat      computeAtAPoint         (_Matrix&m, long row = 0);
+    hyFloat      replaceAPoint           (_Matrix&m, long row, _Matrix&p, hyFloat& nV, _Matrix& fv);
 
     void            ScanAllVariablesOnPartition
     (_SimpleList&, _SimpleList&, _SimpleList&, _SimpleList&, bool = false);
@@ -341,10 +340,10 @@ protected:
     virtual void            ScanAllVariables        (void);
     // internal function to scan all the variables in
 
-    void            OptimalOrder            (long, _SimpleList&);
+    void            OptimalOrder            (long, _SimpleList&, const _SimpleList* clone = nil);
     // determine the optimal order of compuation for a block
 
-    _Parameter      ComputeBlock            (long, _Parameter* siteResults = nil, long currentRateClass = -1, long = -1, _SimpleList* = nil);
+    hyFloat      ComputeBlock            (long, hyFloat* siteResults = nil, long currentRateClass = -1, long = -1, _SimpleList* = nil);
     // 20090224: SLKP
     // added the option to pass an interior branch (referenced by the 3rd argument in the same order as flatTree)
     // and a set of values for each site pattern (indexed left to right) in the 4th argument
@@ -363,24 +362,23 @@ protected:
     void            Clear                   (void);
 
 
-    long            Bracket                 (long , _Parameter& , _Parameter& , _Parameter& ,
-            _Parameter& , _Parameter& , _Parameter& , _Parameter&, _Matrix* = nil);
-    //long          GradientBracketOneVar (_Matrix&, _Matrix& , _Matrix& , _Matrix&,  _Parameter& ,
-    //                                      _Parameter&, _Parameter&, _Parameter&, bool retry = false);
-    void            LocateTheBump         (long,_Parameter , _Parameter& , _Parameter&, _Parameter = -1.);
-    void            GradientLocateTheBump (_Parameter, _Parameter&, _Matrix&, _Matrix&);
-    void            GradientDescent       (_Parameter& , _Matrix& );
-    _Parameter      ConjugateGradientDescent
-    (_Parameter , _Matrix& , bool localOnly = false, long = 0x7fffffff,_SimpleList* only_these_parameters = nil, _Parameter check_lf = A_LARGE_NUMBER);
+    long            Bracket                 (long , hyFloat& , hyFloat& , hyFloat& ,
+            hyFloat& , hyFloat& , hyFloat& , hyFloat&, bool, _Matrix* = nil);
+    //long          GradientBracketOneVar (_Matrix&, _Matrix& , _Matrix& , _Matrix&,  hyFloat& ,
+    //                                      hyFloat&, hyFloat&, hyFloat&, bool retry = false);
+    void            LocateTheBump         (long,hyFloat , hyFloat& , hyFloat&, bool, hyFloat = -1.);
+    hyFloat         GradientLocateTheBump (hyFloat, hyFloat&, _Matrix&, _Matrix&);
+    void            GradientDescent       (hyFloat& , _Matrix& );
+    hyFloat            ConjugateGradientDescent
+    (hyFloat , _Matrix& , bool localOnly = false, long = 0x7fffffff,_SimpleList* only_these_parameters = nil, hyFloat check_lf = -INFINITY);
 
-    _Parameter      SetParametersAndCompute
-    (long, _Parameter, _Matrix* = nil, _Matrix* = nil, bool skip_compute = false);
+    hyFloat      SetParametersAndCompute
+    (long, hyFloat, _Matrix* = nil, _Matrix* = nil,  bool skip_compute = false);
 
     long            CostOfPath            (_DataSetFilter const*, _TheTree const* , _SimpleList&, _SimpleList* = nil) const;
 
     void            BuildLeafProbs        (node<long>& , unsigned long*, unsigned long, _DataSet&, _TheTree*, unsigned long&, bool, long, _DataSetFilter const*, long, _DataSet* = nil) const;
     bool            SingleBuildLeafProbs  (node<long>&, long, _SimpleList&, _SimpleList&, _TheTree*, bool,_DataSetFilter const*, _SimpleList* = nil) const;
-    void            CodonNeutralSimulate  (node<long>&, long, bool,_Matrix*,_Matrix*, _Parameter&, _Parameter&);
 
     bool            HasBlockChanged       (long) const;
     long            BlockLength           (long) const;
@@ -390,8 +388,7 @@ protected:
 
 
     static  void            RandomizeList               (_SimpleList&, long);
-    static  void            CheckFibonacci              (_Parameter);
-
+    static  void            CheckFibonacci              (hyFloat);
 
 
     long            PartitionLengths            (char = 0,  _SimpleList const* = nil);
@@ -410,31 +407,26 @@ protected:
             argument 2 provides an optional subcollection of partititons to operate on;
                 the default is to operate on all
     */
-
-
-protected:
-
-    void            _TerminateAndDump           (const _String& error);
+  
     void            ComputeParameterPenalty     (void);
-    
-    
-    
+
+
     bool            SendOffToMPI                (long);
     void            InitMPIOptimizer            (void);
     void            CleanupMPIOptimizer         (void);
-    void            ComputeBlockInt1            (long,_Parameter&,_TheTree*,_DataSetFilter*, char);
-    void            CheckStep                   (_Parameter&, _Matrix, _Matrix* selection = nil);
-    void            GetGradientStepBound        (_Matrix&, _Parameter &, _Parameter &, long* = nil);
-    void            ComputeGradient             (_Matrix&,  _Parameter&, _Matrix&, _SimpleList&,
+    void            ComputeBlockInt1            (long,hyFloat&,_TheTree*,_DataSetFilter*, char);
+    void            CheckStep                   (hyFloat&, const _Matrix&, _Matrix* selection = nil);
+    void            GetGradientStepBound        (_Matrix&, hyFloat &, hyFloat &, long* = nil);
+    void            ComputeGradient             (_Matrix&,  hyFloat&, _Matrix&, _SimpleList&,
             long, bool normalize = true);
-    bool            SniffAround                 (_Matrix& , _Parameter& , _Parameter&);
-    void            RecurseCategory             (long,long,long,long,_Parameter
+    bool            SniffAround                 (_Matrix& , hyFloat& , hyFloat&);
+    void            RecurseCategory             (long,long,long,long,hyFloat
 #ifdef _SLKP_LFENGINE_REWRITE_
-            ,_SimpleList* = nil, char = 0, _Parameter* = nil,
+            ,_SimpleList* = nil, char = 0, hyFloat* = nil,
             long = -1, _SimpleList* = nil
 #endif
                                                 );
-    void            RecurseConstantOnPartition  (long, long, long, long, _Parameter, _Matrix&);
+    void            RecurseConstantOnPartition  (long, long, long, long, hyFloat, _Matrix&);
 
 
 
@@ -452,12 +444,12 @@ protected:
     */
     void            CleanUpOptimize             (void);
     void            ComputeBlockForTemplate     (long, bool = false);
-    void            ComputeBlockForTemplate2    (long, _Parameter*, _Parameter*, long);
+    void            ComputeBlockForTemplate2    (long, hyFloat*, hyFloat*, long);
     void            DeleteCaches                (bool = true);
     void            PopulateConditionalProbabilities
-    (long index, char runMode, _Parameter* buffer, _SimpleList& scalers, long = -1, _SimpleList* = nil);
+    (long index, char runMode, hyFloat* buffer, _SimpleList& scalers, long = -1, _SimpleList* = nil);
     void            ComputeSiteLikelihoodsForABlock
-    (long, _Parameter*, _SimpleList&, long = -1, _SimpleList* = nil,  char = 0);
+    (long, hyFloat*, _SimpleList&, long = -1, _SimpleList* = nil,  char = 0);
 
     // this function computes a list of site probabilities for the i-th block (1st parameter)
     // stores them in pattern (left to right) order (2nd argument)
@@ -466,7 +458,7 @@ protected:
     // allows the calculation of the probability vector while setting a specific interior branch
     // to a given sequence
 
-    _Parameter          SumUpHiddenMarkov (const _Parameter *, _Matrix&, _Matrix&, _SimpleList const *, const _SimpleList*, long);
+    hyFloat          SumUpHiddenMarkov (const hyFloat *, _Matrix&, _Matrix&, _SimpleList const *, const _SimpleList*, long);
     /*
         SLKP 20090420
 
@@ -482,11 +474,11 @@ protected:
         compute the log likelihood of the partition using the forward HMM algorithm with scaling
      */
 
-    void                    RunViterbi (_Matrix & , const _Parameter * , _Matrix& , _Matrix& , _SimpleList const * ,  const _SimpleList* , long );
+    void                    RunViterbi (_Matrix & , const hyFloat * , _Matrix& , _Matrix& , _SimpleList const * ,  const _SimpleList* , long );
     /* Viterbi decoding for HMM; parameter meanings as in SumUpHiddenMarkov,
        except the first, which will store the optimal path to be returned */
 
-    _Parameter              SumUpSiteLikelihoods        (long, const _Parameter*, const _SimpleList&);
+    hyFloat              SumUpSiteLikelihoods        (long, const hyFloat*, const _SimpleList&);
     /*
      SLKP 20090318
 
@@ -497,17 +489,17 @@ protected:
      compute the log likelihood of the partition
 
      */
-  
+
      /** optimization logger functions **/
-  
-    void LoggerLogL               (_Parameter logL);
-    void LoggerAddGradientPhase   (_Parameter precision);
-    void LoggerAddCoordinatewisePhase (_Parameter shrinkage, char convergence_mode);
+
+    void LoggerLogL               (hyFloat logL);
+    void LoggerAddGradientPhase   (hyFloat precision, hyFloat beta, hyFloat scalar_product);
+    void LoggerAddCoordinatewisePhase (hyFloat shrinkage, char convergence_mode);
     void LoggerAllVariables          ();
-    void LoggerSingleVariable        (unsigned long index, _Parameter logL, _Parameter bracket_precision, _Parameter brent_precision, _Parameter bracket_width, unsigned long bracket_evals, unsigned long brent_evals);
+    void LoggerSingleVariable        (unsigned long index, hyFloat logL, hyFloat bracket_precision, hyFloat brent_precision, hyFloat bracket_width, hyFloat movement, unsigned long bracket_evals, unsigned long brent_evals);
 
 
-    void            UpdateBlockResult           (long, _Parameter);
+    void            UpdateBlockResult           (long, hyFloat);
     /*
         SLKP 20090318
 
@@ -518,8 +510,6 @@ protected:
 
     _List*          RecoverAncestralSequencesMarginal
     (long, _Matrix&,_List const&, bool = false);
-    void            DetermineLocalUpdatePolicy  (void);
-    void            FlushLocalUpdatePolicy      (void);
     void            RestoreScalingFactors       (long, long, long, long*, long *);
     void            SetupLFCaches               (void);
     void            SetupCategoryCaches         (void);
@@ -539,7 +529,7 @@ protected:
     /* 20110718: SLKP this list holds the index of the parameter interval mapping function
         used during optimization */
 
-    _GrowingVector  computationalResults;
+    _Vector  computationalResults;
 
     _List           optimalOrders,
                     leafSkips,
@@ -575,8 +565,8 @@ protected:
                                 _hyphyCategoryCOP
                      */
                     indVarsByPartition,
-                    depVarsByPartition;
-
+                    depVarsByPartition,
+                    *variable_to_node_map;
 
 
     long            evalsSinceLastSetup,
@@ -617,13 +607,12 @@ protected:
         this is used for internal normalization transforms during optimization
      */
 
-    bool            hasBeenOptimized,
-                    siteArrayPopulated;
+    bool            siteArrayPopulated;
 
     _Formula*       computingTemplate;
     MSTCache*       mstCache;
-    
-    _Parameter      smoothingTerm,
+
+    hyFloat         smoothingTerm,
                     smoothingReduction,
                     smoothingPenalty;
 
@@ -654,7 +643,7 @@ protected:
             and read off filterCharDimension characters from there
     */
 
-    _Parameter**        conditionalInternalNodeLikelihoodCaches,
+    hyFloat**        conditionalInternalNodeLikelihoodCaches,
                **     siteScalingFactors,
                **     branchCaches;
 
@@ -671,10 +660,13 @@ protected:
                         canUseReversibleSpeedups,
                         // a partition will be tagged with '1' if its tree has only
                         // time-reversible models
-                        siteScalerBuffer
+                        siteScalerBuffer,
                         // used for LF with category variables to
                         // store site-by-site scaling factors
+                        *_variables_changed_during_last_compute
                         ;
+    
+    _AVLList*           variables_changed_during_last_compute;
 
     _List               localUpdatePolicy,
                         matricesToExponentiate,
@@ -689,7 +681,7 @@ protected:
                         // on a site-by-site basis; includes scratch cache for remapping
                         gradientBlocks
                         ;
-  
+
     _AssociativeList    *optimizatonHistory;
 
 #ifdef  _OPENMP
@@ -701,18 +693,19 @@ protected:
 
 //_______________________________________________________________________________________
 
-class   _CustomFunction: public _LikelihoodFunction
-{
+class   _CustomFunction: public _LikelihoodFunction {
 
 public:
 
-    _CustomFunction         (_String*);
+    _CustomFunction         (const _String& , _VariableContainer const * context = nil);
 
-    virtual     _Parameter  Compute                 (void);
-    virtual     void        RescanAllVariables      (void) {}
-    virtual void            SerializeLF             (_String& res, char=0, _SimpleList* = nil, _SimpleList* = nil) {
-        res.AppendNewInstance ((_String*)myBody.toStr());
+    virtual     hyFloat     Compute                 (void);
+    virtual     void        RescanAllVariables      (bool obtain_variable_mapping = false) {}
+    virtual void            SerializeLF             (_StringBuffer& res, char=0, _SimpleList* = nil, _SimpleList* = nil) {
+               res.AppendNewInstance ((_String*)myBody.toStr(kFormulaStringConversionNormal));
     }
+    
+private:
     _Formula myBody;
 };
 
@@ -723,7 +716,7 @@ extern  bool forceRecomputation,
         isInOptimize;
 
 extern  long lockedLFID;
-
+/*
 extern  _String // declare shared global keys/settings
 globalStartingPoint             ,
 randomStartingPerturbations    ,
@@ -742,13 +735,10 @@ maximumIterationsPerVariable   ,
 optimizationPrecisionMethod    ,
 relativePrecision              ,
 likefuncOutput                 ,
-dataFilePrintFormat                ,
 dataFileDefaultWidth           ,
 dataFileGapWidth               ,
 categorySimulationMethod       ,
 useInitialDistanceGuess            ,
-randomSeed                     ,
-assignedSeed                   ,
 covariancePrecision                ,
 cacheSubtrees                  ,
 likeFuncCountVar               ,
@@ -760,7 +750,6 @@ blockWiseMatrix                    ,
 useFullMST                     ,
 stateCountMatrix               ,
 wStateCountMatrix              ,
-tryNumericSequenceMatch            ,
 allowSequenceMismatch          ,
 shortMPIReturn                 ,
 mpiPrefixCommand               ,
@@ -773,7 +762,7 @@ supportMatrixVariable          ,
 optimizationStatusFile         ,
 autoParalellizeLF              ,
 addLFSmoothing                 ,
-reduceLFSmoothing              ;
+reduceLFSmoothing              ;*/
 
 
 
@@ -782,18 +771,30 @@ void    StateCounterResultHandler   (_Formula&, _SimpleList*,long&,long&,long,_M
 _LikelihoodFunction*
 FindLikeFuncByName           (_String&);
 
+template <typename ACTION>
+void DoForEachLikelihoodFunction (ACTION cb) {
+    for (long i = 0; i < likeFuncNamesList.lLength; i++) {
+        if (((_String*)likeFuncNamesList.GetItem(i))->nonempty()) {
+            cb ((_LikelihoodFunction*)likeFuncList.GetItem (i), i);
+        }
+    }
+}
+
+
 extern  bool                usedCachedResults;
 
-extern _Parameter           _lfScalerUpwards,
+extern hyFloat           _lfScalerUpwards,
        _lfScalingFactorThreshold,
        _logLFScaler;
 
-extern  _GrowingVector      _scalerMultipliers,
+extern  _Vector      _scalerMultipliers,
         _scalerDividers;
 
-_Parameter                  acquireScalerMultiplier (long);
-_Parameter                  myLog                   (_Parameter);
-_Parameter                  mapParameterToInverval  (_Parameter, char, bool);
+hyFloat                  acquireScalerMultiplier (long);
+hyFloat                  myLog                   (hyFloat);
+long                     addScaler               (hyFloat, long, long);
+hyFloat                  mapParameterToInverval  (hyFloat, char, bool);
+hyFloat                  obtainDerivativeCorrection (hyFloat, char);
 
 #ifdef  __HYPHYMPI__
 extern                  _Matrix     resTransferMatrix;
