@@ -28,7 +28,6 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <unistd.h>
 
 #include "global_things.h"
@@ -83,6 +82,7 @@ const char hy_help_message [] =
 "  -i                       interactive mode; causes HyPhy to always prompt the user for analysis options, even when defaults are available\n"
 "  -p                       postprocessor mode; drops HyPhy into an interactive mode where general post-processing scripts can be selected\n"
 "                           upon analysis completion\n\n"
+"  -m                       write diagnostic messages to messages.log\n"
 "optional global arguments:\n"
 "  BASEPATH=directory path  defines the base directory for all path operations (default is pwd)\n"
 "  CPU=integer              if compiled with OpenMP multithreading support, requests this many threads; HyPhy could use fewer than this\n"
@@ -90,6 +90,8 @@ const char hy_help_message [] =
 "  LIBPATH=directory path   defines the directory where HyPhy library files are located (default installed location is /usr/local/lib/hyphy\n"
 "                           or as configured during CMake installation\n"
 "  USEPATH=directory path   specifies the optional working and relative path directory (default is BASEPATH)\n\n"
+"  ENV=expression           set HBL environment variables via explicit statements\n"
+"                           for example ENV='DEBUG_MESSAGES=1;WRITE_LOGS=1'\n"
 "  batch file to run        if specified, execute this file, otherwise drop into an interactive mode\n"
 "  analysis arguments       if batch file is present, all remaining positional arguments are interpreted as inputs to analysis prompts\n\n"
 "optional keyword arguments (can appear anywhere); will be consumed by the requested analysis\n"
@@ -100,22 +102,23 @@ const char hy_help_message [] =
 "Select a standard analysis from the list : \n\thyphy -i \n"
 "Run a standard analysis with default options and one required user argument; \n\thyphy busted --alignment path/to/file\n"
 "Run a standard analysis with additional keyword arguments \n\thyphy busted --alignment path/to/file --srv No\n"
-"See whcih arguments are understood by a standard analysis \n\thyphy busted --help\n"
+"See which arguments are understood by a standard analysis \n\thyphy busted --help\n"
 "Run a custom analysis and pass it some arguments \n\thyphy path/to/hyphy.script argument1 'argument 2' \n"
 ;
 
 const char hy_available_cli_analyses [] =
-"Available standard analyses and their [analysisName] are listed below:\n\n"
-"        [MEME] Test for episodic site-level selection using MEME (Mixed Effects Model of Evolution).\n"
-"        [FEL] Test for pervasive site-level selection using FEL (Fixed Effects Likelihood).\n"
-"        [FUBAR] Test for pervasive site-level selection using FUBAR (Fast Unconstrained Bayesian AppRoximation for inferring selection).\n"
-"        [FADE] Test a protein alignment for directional selection towards specific amino acids along a specified set of test branches using FADE (a FUBAR Approach to Directional Evolution).\n"
-"        [SLAC] Test for pervasive site-level selection using SLAC (Single Likelihood Ancestor Counting).\n"
-"        [BUSTED] Test for episodic gene-wide selection using BUSTED (Branch-site Unrestricted Statistical Test of Episodic Diversification).\n"
-"        [BGM] Apply Bayesian Graphical Model inference to substitution histories at individual sites.\n"
-"        [aBSREL] Test for lineage-specific evolution using the branch-site method aBS-REL (Adaptive Branch-Site Random Effects Likelihood).\n"
-"        [RELAX] Test for relaxation of selection pressure along a specified set of test branches using RELAX (a random effects test of selection relaxation).\n"
-"        [GARD] Screen an alignment for recombination using GARD (Genetic Algorithm for Recombination Detection).\n\n"
+"Available standard analyses and their [standard analysis name] are listed below:\n\n"
+"        [meme] Test for episodic site-level selection using MEME (Mixed Effects Model of Evolution).\n"
+"        [contrast-fel] Use a FEL method to test which sites in a gene may be associated with adaptation to a different environment.\n"
+"        [fel] Test for pervasive site-level selection using FEL (Fixed Effects Likelihood).\n"
+"        [fubar] Test for pervasive site-level selection using FUBAR (Fast Unconstrained Bayesian AppRoximation for inferring selection).\n"
+"        [fade] Test a protein alignment for directional selection towards specific amino acids along a specified set of test branches using FADE (a FUBAR Approach to Directional Evolution).\n"
+"        [slac] Test for pervasive site-level selection using SLAC (Single Likelihood Ancestor Counting).\n"
+"        [busted] Test for episodic gene-wide selection using BUSTED (Branch-site Unrestricted Statistical Test of Episodic Diversification).\n"
+"        [bgm] Apply Bayesian Graphical Model inference to substitution histories at individual sites.\n"
+"        [absrel] Test for lineage-specific evolution using the branch-site method aBS-REL (Adaptive Branch-Site Random Effects Likelihood).\n"
+"        [relax] Test for relaxation of selection pressure along a specified set of test branches using RELAX (a random effects test of selection relaxation).\n"
+"        [gard] Screen an alignment for recombination using GARD (Genetic Algorithm for Recombination Detection).\n\n"
 ;
 
 
@@ -168,7 +171,9 @@ _String baseArgDir,
         libArgDir;
 
 const   _String kLoggedFileEntry ("__USER_ENTRY__"),
-                kHelpKeyword     ("--help");
+                kHelpKeyword     ("--help"),
+                kVersionKeyword  ("--version"),
+                kVerboseKeyword  ("--verbose");
 
 void    ReadInTemplateFiles         (void);
 long    DisplayListOfChoices        (void);
@@ -186,7 +191,8 @@ bool    usePostProcessors = false,
         calculatorMode    = false,
         updateMode       = false,
         pipeMode         = false,
-        logInputMode   = false;
+        logInputMode   = false,
+        displayHelpAndExit = false;
 char    prefFileName[] = ".hyphyinit";
 
 #ifdef  __HYPHYMPI__
@@ -552,16 +558,26 @@ long    DisplayListOfPostChoices (void) {
 }
 
 
-
+void    DisplayHelpMessage (void) {
+   printf ("%s\n%s", hy_usage, hy_help_message);
+   printf ("Available standard keyword analyses (located in %s)\n", getLibraryPath().get_str());
+   TrieIterator options (&availableTemplateFilesAbbreviations);
+   for (TrieIteratorKeyValue ti : options) {
+       printf ("\t%s \t%s\n", ti.get_key().get_str(), ((_String*)(availableTemplateFiles.GetItem(availableTemplateFilesAbbreviations.GetValue(ti.get_value()),1)))->get_str());
+   }
+   printf ("\n");
+   //fprintf( stderr, "%s\n%s\n%s", hy_usage, hy_help_message, hy_available_cli_analyses );
+   exit (0);
+}
             
 void    ProcessConfigStr (_String const & conf) {
     for (unsigned long i=1UL; i<conf.length(); i++) {
         switch (char c = conf.char_at (i)) {
             case 'h':
             case 'H': {
-                fprintf( stderr, "%s\n%s\n%s", hy_usage, hy_help_message, hy_available_cli_analyses );
-                exit (0);
+                displayHelpAndExit = true;
             }
+            break;
 
             case 'p':
           case 'P': {
@@ -595,7 +611,14 @@ void    ProcessConfigStr (_String const & conf) {
               logInputMode = true;
               break;
           }
-          default: {
+
+          case 'm':
+          case 'M': {
+            hy_messages_log_name = "messages.log";
+            break;
+          }
+
+        default: {
               ReportWarning (_String ("Option" ) & _String (c).Enquote() & " is not valid command line option and will be ignored");
           }
         }
@@ -666,9 +689,57 @@ void    SetStatusLineUser   (_String const s) {
 
 //__________________________________________________________________________________
 
+#include <chrono>
+
 #ifndef __UNITTEST__
 int main (int argc, char* argv[]) {
+    
+    /*long    N       = 1000L;
+    long    repeats = 20L;
+    long    density = 0.25 * RAND_MAX;
+    
+    auto start = std::chrono::high_resolution_clock::now();
+    
+    srand(time(0));
+    
+    _Matrix::switchThreshold = atoi (argv[1]);
+    
+    printf ("%ld\n", _Matrix::switchThreshold);
+    
+    for (long i = 0; i < N; i++) {
+        _Matrix test (61,61,false,true);
+        test.ForEachCellNumeric([density] (hyFloat& e, long i, long r, long c) -> void {
+            if (r != c && rand() < density) {
+                e = genrand_real1 ();
+            } else {
+                e = 0.0;
+            }
+        });
+        for (long r = 0; r < 61; r++) {
+            double s = 0.;
+            for (long c = 0; c < 61; c++) {
+                s += test (r, c);
+            }
+            test.theData[r*61+r] = -s;
+        }
+        
+        test.AmISparse();
+        //ObjectToConsole(&test);
+        
+        //printf ("%d\n", test.is_dense());
+        for (long j = 0; j < repeats; j++) {
+            DeleteObject (test.Exponentiate());
+            //return 0;
+        }
+    }
 
+    auto end = std::chrono::high_resolution_clock::now();
+    std::chrono::duration<double> elapsed = end - start;
+    printf ("%15.12g s\n", elapsed.count());
+    
+    return 0;*/
+    
+ 
 #ifdef _COMPARATIVE_LF_DEBUG_DUMP
     FILE * comparative_lf_debug_matrix_content_file = doFileOpen (_COMPARATIVE_LF_DEBUG_DUMP, "w");
 #endif
@@ -683,17 +754,16 @@ int main (int argc, char* argv[]) {
     hy_mpi_node_rank  = rank;
     hy_mpi_node_count = size;
     
-   
     
-    /*if (rank == 0) {
-        int i = 0;
+    /*if (rank > 0) {
+        volatile int i = 0;
         char hostname[256];
         gethostname(hostname, sizeof(hostname));
         printf("PID %d on %s ready for attach\n", getpid(), hostname);
         fflush(stdout);
         while (0 == i)
           sleep(5);
-        printf("PID %d on continuing\n", getpid());
+        printf("PID %d continuing\n", getpid());
     }*/
   
 #endif
@@ -704,7 +774,9 @@ int main (int argc, char* argv[]) {
      if (signal (SIGINT, hyphy_sigterm_handler) == SIG_IGN)
          signal (SIGINT, SIG_IGN);
 #endif
-    
+    //printf ("%e\n", mapParameterToInverval (1.322753E-23, 0x2, false));
+    //exit (0);
+
     /*long read = 0L;
     hyFloat value = 0.0;
     
@@ -739,7 +811,36 @@ int main (int argc, char* argv[]) {
     hy_base_directory = baseDir;
     baseArgDir    = hy_base_directory;
     
-    bool          run_help_message = false;
+    /*long testdim = 61;
+    
+    _Matrix testSQR (testdim,testdim,false,true);
+    hyFloat * stash  = (hyFloat *)MemAllocate(testdim*(1+testdim)*sizeof (hyFloat), false, 64),
+            * stash2 = (hyFloat *)MemAllocate(testdim*(1+testdim)*sizeof (hyFloat), false, 64);
+            
+    for (long i = 0; i < 10; i++) {
+        testSQR.ForEachCellNumeric([&] (hyFloat& value, unsigned long index, unsigned long row, unsigned long column)->void{
+            if (row != column) {
+                value = RandomNumber(0., 0.1);
+            } else {
+                value = 0.;
+            }
+        });
+        testSQR.ForEachCellNumeric([&] (hyFloat value, unsigned long index, unsigned long row, unsigned long column)->void{
+            if (row != column) {
+                testSQR.theData[row*testdim+row] -= value;
+            }
+        });
+        _Matrix cpy (testSQR);
+        cpy.Sqr (stash, false);
+        _Matrix cpy2 (testSQR);
+        cpy2.Sqr (stash2, true);
+        _Matrix direct = testSQR * testSQR;
+        printf ("%d %15.12lg %15.12lg %15.12lg\n", i, (cpy-direct).MaxElement(), (cpy2-direct).MaxElement(), (cpy-cpy2).MaxElement());
+    }
+    
+    exit (1);*/
+    
+    
     
 #ifdef _OPENMP
     system_CPU_count = omp_get_max_threads();
@@ -748,7 +849,7 @@ int main (int argc, char* argv[]) {
     _List positional_arguments;
     _AssociativeList kwargs;
   
-    const _String path_consts [] = {"BASEPATH=", "LIBPATH=", "USEPATH=", "CPU="};
+    const _String path_consts [] = {"BASEPATH=", "LIBPATH=", "USEPATH=", "CPU=", "ENV="};
 
     for (unsigned long i=1UL; i<argc; i++) {
       _String thisArg (argv[i]);
@@ -756,7 +857,15 @@ int main (int argc, char* argv[]) {
       if (thisArg.get_char(0)=='-') { // -[LETTER] arguments
           if (thisArg.get_char (1) == '-') {
               if (thisArg == kHelpKeyword) {
-                  run_help_message = true;
+                  displayHelpAndExit = true;
+                  continue;
+              }
+              if (thisArg == kVersionKeyword) {
+                  StringToConsole(GetVersionString()); NLToConsole();
+                  exit (0);
+              }
+              if (thisArg == kVerboseKeyword) {
+                  force_verbosity_from_cli = true;
                   continue;
               }
               if (i + 1 < argc) {
@@ -796,7 +905,10 @@ int main (int argc, char* argv[]) {
           pathNames&&         &baseDir;
       } else if (thisArg.BeginsWith (path_consts[3])) {
           system_CPU_count  = Maximum (1L, thisArg.Cut(path_consts[3].length(),kStringEnd).to_long());
-      } else
+      } else if (thisArg.BeginsWith (path_consts[4])) {
+          hy_env::cli_env_settings = thisArg.Cut(path_consts[4].length(),kStringEnd);
+      }
+      else
       //argFile = thisArg;
       positional_arguments && &thisArg;
     }
@@ -804,10 +916,9 @@ int main (int argc, char* argv[]) {
     GlobalStartup();
     ReadInTemplateFiles();
     
-    if (positional_arguments.empty () && run_help_message) {
+    if (positional_arguments.empty () && displayHelpAndExit) {
         // --help returns the message below
-        fprintf( stderr, "%s\n%s\n%s", hy_usage, hy_help_message, hy_available_cli_analyses );
-        BufferToConsole("\n");
+        DisplayHelpMessage();
         GlobalShutdown();
         return 0;
     }
@@ -940,7 +1051,7 @@ int main (int argc, char* argv[]) {
             ReadBatchFile (argFile,ex);
         }
 
-        if (run_help_message) {
+        if (displayHelpAndExit) {
 #ifdef __HYPHYMPI__
             if (hy_mpi_node_rank == 0L) {
 #endif
